@@ -73,6 +73,9 @@ enum
 
 enum
 {
+	PREPARE_SOURCE,
+	CAN_REUSE_STREAM,
+	REUSE_STREAM,
 	MISSING_PLUGINS,
 	LAST_SIGNAL
 };
@@ -95,9 +98,12 @@ struct _RBPlayerGstPrivate
 
 	gboolean stream_change_pending;
 	gboolean current_track_finishing;
+	gboolean fake_playing_position;
 
 	gboolean emitted_error;
 
+	gint volume_changed;
+	gint volume_applied;
 	float cur_volume;
 	float replaygain_scale;
 
@@ -119,6 +125,7 @@ tick_timeout (RBPlayerGst *mp)
 				      mp->priv->stream_data,
 				      rb_player_get_time (RB_PLAYER (mp)),
 				      -1);
+		mp->priv->fake_playing_position = FALSE;
 	}
 	return TRUE;
 }
@@ -153,7 +160,7 @@ emit_volume_changed_idle (RBPlayerGst *player)
 }
 
 static void
-volume_notify_cb (GObject *element, GParamSpec *pspec, RBPlayerGst *player)
+volume_notify_cb (GObject *element, GstObject *prop_object, GParamSpec *pspec, RBPlayerGst *player)
 {
 	gdouble v;
 	g_object_get (element, "volume", &v, NULL);
@@ -339,24 +346,14 @@ bus_cb (GstBus *bus, GstMessage *message, RBPlayerGst *mp)
 }
 
 static void
-cdda_got_source_cb (GObject *object, GParamSpec *pspec, char *device)
+source_notify_cb (GObject *object, GParamSpec *pspec, RBPlayerGst *player)
 {
 	GstElement *source;
-
 	g_object_get (object, "source", &source, NULL);
-	rb_debug ("got source %p", source);
-	if (source) {
-		g_signal_handlers_disconnect_by_func (object, cdda_got_source_cb, device);
 
-		g_object_set (G_OBJECT (source), "device", device, NULL);
-		g_free (device);
+	g_signal_emit (player, signals[PREPARE_SOURCE], 0, player->priv->uri, source);
 
-		if (g_object_class_find_property (G_OBJECT_GET_CLASS (source), "paranoia-mode"))
-			g_object_set (G_OBJECT (source), "paranoia-mode", 0, NULL);
-
-		if (g_object_class_find_property (G_OBJECT_GET_CLASS (source), "read-speed"))
-			g_object_set (G_OBJECT (source), "read-speed", 1, NULL);
-	}
+	g_object_unref (source);
 }
 
 static gboolean
@@ -377,9 +374,13 @@ construct_pipeline (RBPlayerGst *mp, GError **error)
 				 G_CALLBACK (about_to_finish_cb),
 				 mp, 0);
 	g_signal_connect_object (G_OBJECT (mp->priv->playbin),
-				 "notify::volume",
+				 "deep-notify::volume",
 				 G_CALLBACK (volume_notify_cb),
 				 mp, 0);
+	g_signal_connect_object (G_OBJECT (mp->priv->playbin),
+				 "notify::source",
+				 G_CALLBACK (source_notify_cb),
+				 mp, 0);
 	if (mp->priv->buffer_size != 0) {
 		g_object_set (mp->priv->playbin, "buffer-size", mp->priv->buffer_size * 1024, NULL);
 	}
@@ -487,8 +488,6 @@ construct_pipeline (RBPlayerGst *mp, GError **error)
 		mp->priv->cur_volume = 0;
 	mp->priv->replaygain_scale = 1.0f;
 
-	rb_player_set_volume (RB_PLAYER (mp), mp->priv->cur_volume);
-
 	rb_debug ("pipeline construction complete");
 	return TRUE;
 }
@@ -577,6 +576,7 @@ set_state_and_wait (RBPlayerGst *player, GstState target, GError **error)
 
 				if (message_from_sink (player->priv->audio_sink, message)) {
 					rb_debug ("got error from sink: %s (%s)", gst_error->message, debug);
+					/* Translators: the parameter here is an error message */
 					g_set_error (error,
 						     RB_PLAYER_ERROR,
 						     RB_PLAYER_ERROR_INTERNAL,
@@ -621,7 +621,7 @@ set_state_and_wait (RBPlayerGst *player, GstState target, GError **error)
 		}
 	}
 
-	if (result == FALSE && *error == NULL) {
+	if (result == FALSE && error != NULL && *error == NULL) {
 		g_set_error (error,
 			     RB_PLAYER_ERROR,
 			     RB_PLAYER_ERROR_GENERAL,
@@ -646,14 +646,23 @@ impl_close (RBPlayer *player, const char *uri, GError **error)
 {
 	RBPlayerGst *mp = RB_PLAYER_GST (player);
 
-	mp->priv->playing = FALSE;
-	mp->priv->buffering = FALSE;
-
 	if ((uri != NULL) && (mp->priv->uri != NULL) && strcmp (mp->priv->uri, uri) == 0) {
 		rb_debug ("URI doesn't match current playing URI; ignoring");
 		return TRUE;
 	}
 
+	if (mp->priv->current_track_finishing) {
+		/* this is a bit of a hack.
+		 * about-to-finish should result in either a track change or nothing.
+		 */
+		rb_debug ("not closing in about-to-finish callback");
+		mp->priv->current_track_finishing = FALSE;
+		return TRUE;
+	}
+
+	mp->priv->playing = FALSE;
+	mp->priv->buffering = FALSE;
+
 	_destroy_stream_data (mp);
 	g_free (mp->priv->uri);
 	g_free (mp->priv->prev_uri);
@@ -714,8 +723,19 @@ impl_opened (RBPlayer *player)
 	return mp->priv->uri != NULL;
 }
 
+static void
+set_playbin_volume (RBPlayerGst *player, float volume)
+{
+	/* ignore the deep-notify we get directly from the sink, as it causes deadlock.
+	 * we still get another one anyway.
+	 */
+	g_signal_handlers_block_by_func (player->priv->playbin, volume_notify_cb, player);
+	g_object_set (player->priv->playbin, "volume", volume, NULL);
+	g_signal_handlers_unblock_by_func (player->priv->playbin, volume_notify_cb, player);
+}
+
 static gboolean
-impl_play (RBPlayer *player, gint crossfade, GError **error)
+impl_play (RBPlayer *player, RBPlayerPlayType play_type, gint64 crossfade, GError **error)
 {
 	RBPlayerGst *mp = RB_PLAYER_GST (player);
 	gboolean result;
@@ -729,66 +749,43 @@ impl_play (RBPlayer *player, gint crossfade, GError **error)
 	} else if (mp->priv->current_track_finishing) {
 		rb_debug ("current track finishing -> just setting URI on playbin");
 		g_object_set (mp->priv->playbin, "uri", mp->priv->uri, NULL);
-
-		_rb_player_emit_playing_stream (RB_PLAYER (mp), mp->priv->stream_data);
-
 		result = TRUE;
+
+		/* since we don't know exactly when the change occurs, pretend the playing position
+		 * is 0 until the next tick.  otherwise we sometimes get the playing position from the
+		 * previous track.
+		 */
+		mp->priv->fake_playing_position = TRUE;
+
 	} else {
-		gboolean cdda_seek = FALSE;
+		gboolean reused = FALSE;
 
-		rb_debug ("not in transition, stopping current track to start the new one");
+		/* try to reuse the stream */
+		if (mp->priv->prev_uri != NULL) {
+			g_signal_emit (mp,
+				       signals[CAN_REUSE_STREAM], 0,
+				       mp->priv->uri, mp->priv->prev_uri, mp->priv->playbin,
+				       &reused);
 
-		/* check if we are switching tracks on a cd, so we don't have to close the device */
-		if (g_str_has_prefix (mp->priv->uri, "cdda://")) {
-			const char *old_device = NULL;
-			const char *new_device;
-
-			if (mp->priv->prev_uri && g_str_has_prefix (mp->priv->prev_uri, "cdda://"))
-				old_device = g_utf8_strchr (mp->priv->prev_uri, -1, '#');
-
-			new_device = g_utf8_strchr (mp->priv->uri, -1, '#');
-
-			if (old_device && strcmp (old_device, new_device) == 0) {
-				GstFormat track_format = gst_format_get_by_nick ("track");
-				char *track_str;
-				guint track;
-				guint cdda_len;
-
-				cdda_len = strlen ("cdda://");
-				track_str = g_strndup (mp->priv->uri + cdda_len, new_device - (mp->priv->uri + cdda_len));
-				track = atoi (track_str);
-				g_free (track_str);
-
-				rb_debug ("seeking to track %d on CD device %s", track, new_device);
-				if (gst_element_seek (mp->priv->playbin, 1.0,
-						      track_format, GST_SEEK_FLAG_FLUSH,
-						      GST_SEEK_TYPE_SET, track - 1,
-						      GST_SEEK_TYPE_NONE, -1)) {
-					cdda_seek = TRUE;
-					result = TRUE;
-					_rb_player_emit_playing_stream (RB_PLAYER (mp), mp->priv->stream_data);
-				}
-			} else {
-				/* +1 to skip the '#' */
-				char *device = g_strdup (new_device + 1);
-
-				rb_debug ("waiting for source element for CD device %s", device);
-				g_signal_connect (G_OBJECT (mp->priv->playbin),
-						  "notify::source",
-						  G_CALLBACK (cdda_got_source_cb),
-						  device);
+			if (reused) {
+				rb_debug ("reusing stream to switch from %s to %s", mp->priv->prev_uri, mp->priv->uri);
+				g_signal_emit (player,
+					       signals[REUSE_STREAM], 0,
+					       mp->priv->uri, mp->priv->prev_uri, mp->priv->playbin);
+				result = TRUE;
 			}
 		}
 
-		if (cdda_seek == FALSE) {
+		/* no stream reuse, so stop, set the new URI, then start */
+		if (reused == FALSE) {
+			rb_debug ("not in transition, stopping current track to start the new one");
 			result = set_state_and_wait (mp, GST_STATE_READY, error);
 			if (result == TRUE) {
 				g_object_set (mp->priv->playbin, "uri", mp->priv->uri, NULL);
-
-				_rb_player_emit_playing_stream (RB_PLAYER (mp), mp->priv->stream_data);
 				result = set_state_and_wait (mp, GST_STATE_PLAYING, error);
 			}
 		}
+
 	}
 
 	mp->priv->stream_change_pending = FALSE;
@@ -798,12 +795,36 @@ impl_play (RBPlayer *player, gint crossfade, GError **error)
 		mp->priv->buffering = FALSE;
 		mp->priv->playing = TRUE;
 
+		_rb_player_emit_playing_stream (RB_PLAYER (mp), mp->priv->stream_data);
+
 		if (mp->priv->tick_timeout_id == 0) {
 			mp->priv->tick_timeout_id =
 				g_timeout_add (1000 / RB_PLAYER_GST_TICK_HZ,
 					       (GSourceFunc) tick_timeout,
 					       mp);
 		}
+
+		if (mp->priv->volume_applied == 0) {
+			GstElement *e;
+
+			/* if the sink provides volume control, ignore the first
+			 * volume setting, allowing the sink to restore its own
+			 * volume.
+			 */
+			e = rb_player_gst_find_element_with_property (mp->priv->audio_sink, "volume");
+			if (e != NULL) {
+				mp->priv->volume_applied = 1;
+			}
+			gst_object_unref (e);
+
+			if (mp->priv->volume_applied < mp->priv->volume_changed) {
+				float volume = mp->priv->cur_volume * mp->priv->replaygain_scale;
+				rb_debug ("applying initial volume: %f", volume);
+				set_playbin_volume (mp, volume);
+			}
+
+			mp->priv->volume_applied = mp->priv->volume_changed;
+		}
 	}
 
 	return result;
@@ -892,8 +913,12 @@ impl_set_volume (RBPlayer *player,
 	RBPlayerGst *mp = RB_PLAYER_GST (player);
 	g_return_if_fail (volume >= 0.0 && volume <= 1.0);
 
-	if (mp->priv->playbin != NULL) {
-		g_object_set (mp->priv->playbin, "volume", volume * mp->priv->replaygain_scale, NULL);
+	mp->priv->volume_changed++;
+	if (mp->priv->volume_applied > 0) {
+		set_playbin_volume (mp, volume * mp->priv->replaygain_scale);
+		mp->priv->volume_applied = mp->priv->volume_changed;
+	} else {
+		/* volume will be applied in the first call to impl_play */
 	}
 
 	mp->priv->cur_volume = volume;
@@ -932,7 +957,7 @@ impl_seekable (RBPlayer *player)
 }
 
 static void
-impl_set_time (RBPlayer *player, long time)
+impl_set_time (RBPlayer *player, gint64 time)
 {
 	RBPlayerGst *mp = RB_PLAYER_GST (player);
 	GError *error = NULL;
@@ -950,7 +975,7 @@ impl_set_time (RBPlayer *player, long time)
 
 	gst_element_seek (mp->priv->playbin, 1.0,
 			  GST_FORMAT_TIME, GST_SEEK_FLAG_FLUSH,
-			  GST_SEEK_TYPE_SET, time * GST_SECOND,
+			  GST_SEEK_TYPE_SET, time,
 			  GST_SEEK_TYPE_NONE, -1);
 
 	if (mp->priv->playing) {
@@ -961,20 +986,20 @@ impl_set_time (RBPlayer *player, long time)
 	}
 }
 
-static long
+static gint64
 impl_get_time (RBPlayer *player)
 {
 	RBPlayerGst *mp = RB_PLAYER_GST (player);
 
-	if (mp->priv->playbin != NULL) {
+	if (mp->priv->fake_playing_position) {
+		/* track transition occurring, so pretend we're at the start of the new one */
+		return 0;
+	} else if (mp->priv->playbin != NULL) {
 		gint64 position = -1;
 		GstFormat fmt = GST_FORMAT_TIME;
 
 		gst_element_query_position (mp->priv->playbin, &fmt, &position);
-		if (position != -1)
-			position /= GST_SECOND;
-
-		return (long) position;
+		return position;
 	} else {
 		return -1;
 	}
@@ -1399,6 +1424,36 @@ rb_player_gst_class_init (RBPlayerGstClass *klass)
 							    64, MAX_NETWORK_BUFFER_SIZE, 128,
 							    G_PARAM_READWRITE));
 
+	signals[PREPARE_SOURCE] =
+		g_signal_new ("prepare-source",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (RBPlayerGstClass, prepare_source),
+			      NULL, NULL,
+			      rb_marshal_VOID__STRING_OBJECT,
+			      G_TYPE_NONE,
+			      2,
+			      G_TYPE_STRING, GST_TYPE_ELEMENT);
+	signals[CAN_REUSE_STREAM] =
+		g_signal_new ("can-reuse-stream",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (RBPlayerGstClass, can_reuse_stream),
+			      NULL, NULL,
+			      rb_marshal_BOOLEAN__STRING_STRING_OBJECT,
+			      G_TYPE_BOOLEAN,
+			      3,
+			      G_TYPE_STRING, G_TYPE_STRING, GST_TYPE_ELEMENT);
+	signals[REUSE_STREAM] =
+		g_signal_new ("reuse-stream",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (RBPlayerGstClass, reuse_stream),
+			      NULL, NULL,
+			      rb_marshal_VOID__STRING_STRING_OBJECT,
+			      G_TYPE_NONE,
+			      3,
+			      G_TYPE_STRING, G_TYPE_STRING, GST_TYPE_ELEMENT);
 	signals[MISSING_PLUGINS] =
 		g_signal_new ("missing-plugins",
 			      G_OBJECT_CLASS_TYPE (object_class),
