@@ -198,7 +198,15 @@ extern void rb_marshal_VOID__INT_INT_DOUBLE (GClosure     *closure,
                                              gpointer      invocation_hint,
                                              gpointer      marshal_data);
 
-/* VOID:OBJECT,INT,INT (./rb-marshal.list:25) */
+/* VOID:INT64 (./rb-marshal.list:25) */
+extern void rb_marshal_VOID__INT64 (GClosure     *closure,
+                                    GValue       *return_value,
+                                    guint         n_param_values,
+                                    const GValue *param_values,
+                                    gpointer      invocation_hint,
+                                    gpointer      marshal_data);
+
+/* VOID:OBJECT,INT,INT (./rb-marshal.list:26) */
 extern void rb_marshal_VOID__OBJECT_INT_INT (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -206,7 +214,7 @@ extern void rb_marshal_VOID__OBJECT_INT_INT (GClosure     *closure,
                                              gpointer      invocation_hint,
                                              gpointer      marshal_data);
 
-/* VOID:OBJECT,INT,INT,BOXED,UINT,UINT (./rb-marshal.list:26) */
+/* VOID:OBJECT,INT,INT,BOXED,UINT,UINT (./rb-marshal.list:27) */
 extern void rb_marshal_VOID__OBJECT_INT_INT_BOXED_UINT_UINT (GClosure     *closure,
                                                              GValue       *return_value,
                                                              guint         n_param_values,
@@ -214,7 +222,7 @@ extern void rb_marshal_VOID__OBJECT_INT_INT_BOXED_UINT_UINT (GClosure     *closu
                                                              gpointer      invocation_hint,
                                                              gpointer      marshal_data);
 
-/* VOID:OBJECT,INT,POINTER (./rb-marshal.list:27) */
+/* VOID:OBJECT,INT,POINTER (./rb-marshal.list:28) */
 extern void rb_marshal_VOID__OBJECT_INT_POINTER (GClosure     *closure,
                                                  GValue       *return_value,
                                                  guint         n_param_values,
@@ -222,7 +230,7 @@ extern void rb_marshal_VOID__OBJECT_INT_POINTER (GClosure     *closure,
                                                  gpointer      invocation_hint,
                                                  gpointer      marshal_data);
 
-/* VOID:POINTER,INT (./rb-marshal.list:28) */
+/* VOID:POINTER,INT (./rb-marshal.list:29) */
 extern void rb_marshal_VOID__POINTER_INT (GClosure     *closure,
                                           GValue       *return_value,
                                           guint         n_param_values,
@@ -230,7 +238,15 @@ extern void rb_marshal_VOID__POINTER_INT (GClosure     *closure,
                                           gpointer      invocation_hint,
                                           gpointer      marshal_data);
 
-/* VOID:POINTER,INT,POINTER (./rb-marshal.list:29) */
+/* VOID:POINTER,INT64,INT64 (./rb-marshal.list:30) */
+extern void rb_marshal_VOID__POINTER_INT64_INT64 (GClosure     *closure,
+                                                  GValue       *return_value,
+                                                  guint         n_param_values,
+                                                  const GValue *param_values,
+                                                  gpointer      invocation_hint,
+                                                  gpointer      marshal_data);
+
+/* VOID:POINTER,INT,POINTER (./rb-marshal.list:31) */
 extern void rb_marshal_VOID__POINTER_INT_POINTER (GClosure     *closure,
                                                   GValue       *return_value,
                                                   guint         n_param_values,
@@ -238,7 +254,7 @@ extern void rb_marshal_VOID__POINTER_INT_POINTER (GClosure     *closure,
                                                   gpointer      invocation_hint,
                                                   gpointer      marshal_data);
 
-/* VOID:POINTER,LONG,LONG (./rb-marshal.list:30) */
+/* VOID:POINTER,LONG,LONG (./rb-marshal.list:32) */
 extern void rb_marshal_VOID__POINTER_LONG_LONG (GClosure     *closure,
                                                 GValue       *return_value,
                                                 guint         n_param_values,
@@ -246,7 +262,7 @@ extern void rb_marshal_VOID__POINTER_LONG_LONG (GClosure     *closure,
                                                 gpointer      invocation_hint,
                                                 gpointer      marshal_data);
 
-/* VOID:POINTER,OBJECT (./rb-marshal.list:31) */
+/* VOID:POINTER,OBJECT (./rb-marshal.list:33) */
 extern void rb_marshal_VOID__POINTER_OBJECT (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -254,7 +270,7 @@ extern void rb_marshal_VOID__POINTER_OBJECT (GClosure     *closure,
                                              gpointer      invocation_hint,
                                              gpointer      marshal_data);
 
-/* VOID:POINTER,POINTER (./rb-marshal.list:32) */
+/* VOID:POINTER,POINTER (./rb-marshal.list:34) */
 extern void rb_marshal_VOID__POINTER_POINTER (GClosure     *closure,
                                               GValue       *return_value,
                                               guint         n_param_values,
@@ -262,7 +278,7 @@ extern void rb_marshal_VOID__POINTER_POINTER (GClosure     *closure,
                                               gpointer      invocation_hint,
                                               gpointer      marshal_data);
 
-/* VOID:POINTER,POINTER,POINTER (./rb-marshal.list:33) */
+/* VOID:POINTER,POINTER,POINTER (./rb-marshal.list:35) */
 extern void rb_marshal_VOID__POINTER_POINTER_POINTER (GClosure     *closure,
                                                       GValue       *return_value,
                                                       guint         n_param_values,
@@ -270,7 +286,7 @@ extern void rb_marshal_VOID__POINTER_POINTER_POINTER (GClosure     *closure,
                                                       gpointer      invocation_hint,
                                                       gpointer      marshal_data);
 
-/* VOID:POINTER,UINT (./rb-marshal.list:34) */
+/* VOID:POINTER,UINT (./rb-marshal.list:36) */
 extern void rb_marshal_VOID__POINTER_UINT (GClosure     *closure,
                                            GValue       *return_value,
                                            guint         n_param_values,
@@ -278,7 +294,7 @@ extern void rb_marshal_VOID__POINTER_UINT (GClosure     *closure,
                                            gpointer      invocation_hint,
                                            gpointer      marshal_data);
 
-/* VOID:POINTER,ULONG (./rb-marshal.list:35) */
+/* VOID:POINTER,ULONG (./rb-marshal.list:37) */
 extern void rb_marshal_VOID__POINTER_ULONG (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -286,7 +302,7 @@ extern void rb_marshal_VOID__POINTER_ULONG (GClosure     *closure,
                                             gpointer      invocation_hint,
                                             gpointer      marshal_data);
 
-/* VOID:STRING,DOUBLE (./rb-marshal.list:36) */
+/* VOID:STRING,DOUBLE (./rb-marshal.list:38) */
 extern void rb_marshal_VOID__STRING_DOUBLE (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -294,7 +310,7 @@ extern void rb_marshal_VOID__STRING_DOUBLE (GClosure     *closure,
                                             gpointer      invocation_hint,
                                             gpointer      marshal_data);
 
-/* VOID:STRING,INT (./rb-marshal.list:37) */
+/* VOID:STRING,INT (./rb-marshal.list:39) */
 extern void rb_marshal_VOID__STRING_INT (GClosure     *closure,
                                          GValue       *return_value,
                                          guint         n_param_values,
@@ -302,7 +318,15 @@ extern void rb_marshal_VOID__STRING_INT (GClosure     *closure,
                                          gpointer      invocation_hint,
                                          gpointer      marshal_data);
 
-/* VOID:STRING,POINTER (./rb-marshal.list:38) */
+/* VOID:STRING,OBJECT (./rb-marshal.list:40) */
+extern void rb_marshal_VOID__STRING_OBJECT (GClosure     *closure,
+                                            GValue       *return_value,
+                                            guint         n_param_values,
+                                            const GValue *param_values,
+                                            gpointer      invocation_hint,
+                                            gpointer      marshal_data);
+
+/* VOID:STRING,POINTER (./rb-marshal.list:41) */
 extern void rb_marshal_VOID__STRING_POINTER (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -310,7 +334,7 @@ extern void rb_marshal_VOID__STRING_POINTER (GClosure     *closure,
                                              gpointer      invocation_hint,
                                              gpointer      marshal_data);
 
-/* VOID:STRING,STRING (./rb-marshal.list:39) */
+/* VOID:STRING,STRING (./rb-marshal.list:42) */
 extern void rb_marshal_VOID__STRING_STRING (GClosure     *closure,
                                             GValue       *return_value,
                                             guint         n_param_values,
@@ -318,7 +342,7 @@ extern void rb_marshal_VOID__STRING_STRING (GClosure     *closure,
                                             gpointer      invocation_hint,
                                             gpointer      marshal_data);
 
-/* VOID:STRING,STRING,OBJECT (./rb-marshal.list:40) */
+/* VOID:STRING,STRING,OBJECT (./rb-marshal.list:43) */
 extern void rb_marshal_VOID__STRING_STRING_OBJECT (GClosure     *closure,
                                                    GValue       *return_value,
                                                    guint         n_param_values,
@@ -326,7 +350,7 @@ extern void rb_marshal_VOID__STRING_STRING_OBJECT (GClosure     *closure,
                                                    gpointer      invocation_hint,
                                                    gpointer      marshal_data);
 
-/* VOID:STRING,STRING,STRING (./rb-marshal.list:41) */
+/* VOID:STRING,STRING,STRING (./rb-marshal.list:44) */
 extern void rb_marshal_VOID__STRING_STRING_STRING (GClosure     *closure,
                                                    GValue       *return_value,
                                                    guint         n_param_values,
@@ -334,7 +358,7 @@ extern void rb_marshal_VOID__STRING_STRING_STRING (GClosure     *closure,
                                                    gpointer      invocation_hint,
                                                    gpointer      marshal_data);
 
-/* VOID:STRING,STRING,STRING,UINT,BOOLEAN (./rb-marshal.list:42) */
+/* VOID:STRING,STRING,STRING,UINT,BOOLEAN (./rb-marshal.list:45) */
 extern void rb_marshal_VOID__STRING_STRING_STRING_UINT_BOOLEAN (GClosure     *closure,
                                                                 GValue       *return_value,
                                                                 guint         n_param_values,
@@ -342,7 +366,7 @@ extern void rb_marshal_VOID__STRING_STRING_STRING_UINT_BOOLEAN (GClosure     *cl
                                                                 gpointer      invocation_hint,
                                                                 gpointer      marshal_data);
 
-/* VOID:UINT,STRING,STRING,OBJECT,BOOLEAN (./rb-marshal.list:43) */
+/* VOID:UINT,STRING,STRING,OBJECT,BOOLEAN (./rb-marshal.list:46) */
 extern void rb_marshal_VOID__UINT_STRING_STRING_OBJECT_BOOLEAN (GClosure     *closure,
                                                                 GValue       *return_value,
                                                                 guint         n_param_values,
@@ -350,7 +374,7 @@ extern void rb_marshal_VOID__UINT_STRING_STRING_OBJECT_BOOLEAN (GClosure     *cl
                                                                 gpointer      invocation_hint,
                                                                 gpointer      marshal_data);
 
-/* VOID:ULONG,FLOAT (./rb-marshal.list:44) */
+/* VOID:ULONG,FLOAT (./rb-marshal.list:47) */
 extern void rb_marshal_VOID__ULONG_FLOAT (GClosure     *closure,
                                           GValue       *return_value,
                                           guint         n_param_values,
@@ -358,7 +382,7 @@ extern void rb_marshal_VOID__ULONG_FLOAT (GClosure     *closure,
                                           gpointer      invocation_hint,
                                           gpointer      marshal_data);
 
-/* VOID:OBJECT,BOOLEAN (./rb-marshal.list:45) */
+/* VOID:OBJECT,BOOLEAN (./rb-marshal.list:48) */
 extern void rb_marshal_VOID__OBJECT_BOOLEAN (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
@@ -366,7 +390,7 @@ extern void rb_marshal_VOID__OBJECT_BOOLEAN (GClosure     *closure,
                                              gpointer      invocation_hint,
                                              gpointer      marshal_data);
 
-/* VOID:STRING,STRING,POINTER,POINTER (./rb-marshal.list:46) */
+/* VOID:STRING,STRING,POINTER,POINTER (./rb-marshal.list:49) */
 extern void rb_marshal_VOID__STRING_STRING_POINTER_POINTER (GClosure     *closure,
                                                             GValue       *return_value,
                                                             guint         n_param_values,
