@@ -7,7 +7,7 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  The Rhythmbox authors hereby grants permission for non-GPL compatible
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
  *  GStreamer plugins to be used and distributed together with GStreamer
  *  and Rhythmbox. This permission is above and beyond the permissions granted
  *  by the GPL license by which Rhythmbox is covered. If you modify this code
@@ -33,15 +33,16 @@
 #include <totem-pl-parser.h>
 
 #include "rb-generic-player-playlist-source.h"
+#include "rb-generic-player-source.h"
 #include "rb-debug.h"
 #include "rb-plugin.h"
 #include "rb-file-helpers.h"
 
-#define PLAYLIST_SAVE_TIMEOUT	1000
+#define PLAYLIST_SAVE_TIMEOUT	1
 
 typedef struct
 {
-	char *playlist_path;
+	char *playlist_path;		/* hmm, replace with GFile? */
 	char *device_root;
 	gint save_playlist_id;
 	RBGenericPlayerSource *player_source;
@@ -63,7 +64,7 @@ enum {
 
 
 static void
-rb_generic_player_playlist_source_save_to_xml (RBPlaylistSource *source, xmlNodePtr node)
+impl_save_to_xml (RBPlaylistSource *source, xmlNodePtr node)
 {
 	/* do nothing; just to prevent weirdness */
 }
@@ -138,7 +139,8 @@ save_playlist (RBGenericPlayerPlaylistSource *source)
 		char *mount_uri;
 		char *filename;
 		const char *ext;
-		GnomeVFSURI *uri, *nuri;
+		GFile *dir;
+		GFile *playlist;
 
 		ext = playlist_format_extension (playlist_type);
 
@@ -152,21 +154,22 @@ save_playlist (RBGenericPlayerPlaylistSource *source)
 		playlist_dir = rb_generic_player_source_get_playlist_path (priv->player_source);
 		mount_uri = rb_generic_player_source_get_mount_path (priv->player_source);
 
-		uri = gnome_vfs_uri_new (mount_uri);
+		dir = g_file_new_for_uri (mount_uri);
 		if (playlist_dir != NULL) {
-			nuri = gnome_vfs_uri_append_path (uri, playlist_dir);
-			gnome_vfs_uri_unref (uri);
-			uri = nuri;
+			GFile *pdir;
+
+			pdir = g_file_resolve_relative_path (dir, playlist_dir);
+			g_object_unref (dir);
+			dir = pdir;
 		}
 
-		nuri = gnome_vfs_uri_append_path (uri, filename);
-		gnome_vfs_uri_unref (uri);
+		playlist = g_file_resolve_relative_path (dir, filename);
+		priv->playlist_path = g_file_get_path (playlist);
 		
 		g_free (mount_uri);
 		g_free (playlist_dir);
 
-		priv->playlist_path = gnome_vfs_uri_to_string (nuri, GNOME_VFS_URI_HIDE_NONE);
-		gnome_vfs_uri_unref (nuri);
+		g_object_unref (dir);
 	}
 
 	temp_uri = g_strdup_printf ("%s%06X", priv->playlist_path, g_random_int_range (0, 0xFFFFFF));
@@ -186,14 +189,19 @@ save_playlist (RBGenericPlayerPlaylistSource *source)
 		/* XXX report this more usefully */
 		g_warning ("Playlist save failed: %s", error->message);
 	} else {
-		/* rename the new file over the old one */
-		GnomeVFSResult result;
+		GFile *dest;
+		GFile *src;
 
-		result = gnome_vfs_move (temp_uri, priv->playlist_path, TRUE);
-		if (result != GNOME_VFS_OK) {
+		dest = g_file_new_for_path (priv->playlist_path);
+		src = g_file_new_for_path (temp_uri);
+		g_file_move (src, dest, G_FILE_COPY_OVERWRITE | G_FILE_COPY_NO_FALLBACK_FOR_MOVE, NULL, NULL, NULL, &error);
+		if (error != NULL) {
 			/* XXX report this more usefully */
-			g_warning ("Replacing playlist failed: %s", gnome_vfs_result_to_string (result));
+			g_warning ("Replacing playlist failed: %s", error->message);
 		}
+
+		g_object_unref (dest);
+		g_object_unref (src);
 	}
 
 	g_clear_error (&error);
@@ -249,7 +257,9 @@ load_playlist (RBGenericPlayerPlaylistSource *source)
 	RBGenericPlayerPlaylistSourcePrivate *priv = GET_PRIVATE (source);
 	TotemPlParser *parser;
 	gboolean result;
-	char *path;
+	GFile *file;
+	char *name;
+	char *uri;
 
 	if (priv->playlist_path == NULL) {
 		/* this happens when we're creating a new playlist */
@@ -259,16 +269,12 @@ load_playlist (RBGenericPlayerPlaylistSource *source)
 	}
 
 	priv->loading = TRUE;
+	file = g_file_new_for_path (priv->playlist_path);
 
 	/* make a default name for the playlist based on the filename */
-	path = g_filename_from_uri (priv->playlist_path, NULL, NULL);
-	if (path != NULL) {
-		char *name = g_path_get_basename (path);
-
-		g_object_set (source, "name", name, NULL);
-		g_free (name);
-		g_free (path);
-	}
+	name = g_file_get_basename (file);
+	g_object_set (source, "name", name, NULL);
+	g_free (name);
 
 	parser = totem_pl_parser_new ();
 	if (rb_debug_matches ("totem_pl_parser_parse", "totem-pl-parser.c")) {
@@ -284,7 +290,8 @@ load_playlist (RBGenericPlayerPlaylistSource *source)
 			  source);
 	g_object_set (G_OBJECT (parser), "recurse", FALSE, NULL);
 
-	switch (totem_pl_parser_parse_with_base (parser, priv->playlist_path, priv->device_root, FALSE)) {
+	uri = g_file_get_uri (file);
+	switch (totem_pl_parser_parse_with_base (parser, uri, priv->device_root, FALSE)) {
 	case TOTEM_PL_PARSER_RESULT_SUCCESS:
 		rb_debug ("playlist parsed successfully");
 		result = TRUE;
@@ -307,13 +314,15 @@ load_playlist (RBGenericPlayerPlaylistSource *source)
 	default:
 		g_assert_not_reached ();
 	}
+	g_free (uri);
+	g_object_unref (file);
 
 	priv->loading = FALSE;
 	return result;
 }
 
 static void
-rb_generic_player_playlist_source_mark_dirty (RBPlaylistSource *source)
+impl_mark_dirty (RBPlaylistSource *source)
 {
 	RBGenericPlayerPlaylistSourcePrivate *priv = GET_PRIVATE (source);
 
@@ -325,9 +334,9 @@ rb_generic_player_playlist_source_mark_dirty (RBPlaylistSource *source)
 		g_source_remove (priv->save_playlist_id);
 	}
 
-	priv->save_playlist_id = g_timeout_add (PLAYLIST_SAVE_TIMEOUT,
-						(GSourceFunc) save_playlist,
-						source);
+	priv->save_playlist_id = g_timeout_add_seconds (PLAYLIST_SAVE_TIMEOUT,
+							(GSourceFunc) save_playlist,
+							source);
 }
 
 RBSource *
@@ -354,12 +363,16 @@ rb_generic_player_playlist_delete_from_player (RBGenericPlayerPlaylistSource *so
 	RBGenericPlayerPlaylistSourcePrivate *priv = GET_PRIVATE (source);
 
 	if (priv->playlist_path != NULL) {
-		GnomeVFSResult result;
-		result = gnome_vfs_unlink (priv->playlist_path);
+		GError *error = NULL;
+		GFile *playlist;
 
-		if (result != GNOME_VFS_OK) {
-			/* now what? */
+		playlist = g_file_new_for_path (priv->playlist_path);
+		g_file_delete (playlist, NULL, &error);
+		if (error != NULL) {
+			g_warning ("Deleting playlist %s failed: %s", priv->playlist_path, error->message);
+			g_clear_error (&error);
 		}
+		g_object_unref (playlist);
 	} else {
 		rb_debug ("playlist was never saved: nothing to delete");
 	}
@@ -371,7 +384,7 @@ rb_generic_player_playlist_source_init (RBGenericPlayerPlaylistSource *source)
 }
 
 static GObject *
-rb_generic_player_playlist_source_constructor (GType type, guint n_construct_properties, GObjectConstructParam *construct_properties)
+impl_constructor (GType type, guint n_construct_properties, GObjectConstructParam *construct_properties)
 {
 	RBGenericPlayerPlaylistSource *source;
 
@@ -389,7 +402,7 @@ rb_generic_player_playlist_source_constructor (GType type, guint n_construct_pro
 }
 
 static void
-rb_generic_player_playlist_source_dispose (GObject *object)
+impl_dispose (GObject *object)
 {
 	RBGenericPlayerPlaylistSourcePrivate *priv = GET_PRIVATE (object);
 
@@ -407,7 +420,7 @@ rb_generic_player_playlist_source_dispose (GObject *object)
 }
 
 static void
-rb_generic_player_playlist_source_finalize (GObject *object)
+impl_finalize (GObject *object)
 {
 	RBGenericPlayerPlaylistSourcePrivate *priv = GET_PRIVATE (object);
 
@@ -417,7 +430,7 @@ rb_generic_player_playlist_source_finalize (GObject *object)
 }
 
 static void
-rb_generic_player_playlist_source_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
+impl_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec)
 {
 	RBGenericPlayerPlaylistSourcePrivate *priv = GET_PRIVATE (object);
 
@@ -438,7 +451,7 @@ rb_generic_player_playlist_source_get_property (GObject *object, guint prop_id,
 }
 
 static void
-rb_generic_player_playlist_source_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
+impl_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec)
 {
 	RBGenericPlayerPlaylistSourcePrivate *priv = GET_PRIVATE (object);
 
@@ -459,12 +472,47 @@ rb_generic_player_playlist_source_set_property (GObject *object, guint prop_id,
 }
 
 static gboolean
-rb_generic_player_playlist_source_show_popup (RBSource *source)
+impl_show_popup (RBSource *source)
 {
 	_rb_source_show_popup (source, "/GenericPlayerPlaylistSourcePopup");
 	return TRUE;
 }
 
+static gboolean
+impl_can_move_to_trash (RBSource *source)
+{
+	RBGenericPlayerPlaylistSourcePrivate *priv = GET_PRIVATE (source);
+	RBEntryView *songs;
+	GList *sel;
+	gboolean ret;
+
+	songs = rb_source_get_entry_view (source);
+	sel = rb_entry_view_get_selected_entries (songs);
+
+	ret = rb_generic_player_source_can_trash_entries (priv->player_source, sel);
+
+	g_list_foreach (sel, (GFunc) rhythmdb_entry_unref, NULL);
+	g_list_free (sel);
+
+	return ret;
+}
+
+static void
+impl_move_to_trash (RBSource *source)
+{
+	RBGenericPlayerPlaylistSourcePrivate *priv = GET_PRIVATE (source);
+	RBEntryView *songs;
+	GList *sel;
+
+	songs = rb_source_get_entry_view (source);
+	sel = rb_entry_view_get_selected_entries (songs);
+
+	rb_generic_player_source_trash_or_delete_entries (priv->player_source, sel, FALSE);
+
+	g_list_foreach (sel, (GFunc) rhythmdb_entry_unref, NULL);
+	g_list_free (sel);
+}
+
 static void
 rb_generic_player_playlist_source_class_init (RBGenericPlayerPlaylistSourceClass *klass)
 {
@@ -472,16 +520,18 @@ rb_generic_player_playlist_source_class_init (RBGenericPlayerPlaylistSourceClass
 	RBSourceClass *source_class = RB_SOURCE_CLASS (klass);
 	RBPlaylistSourceClass *playlist_class = RB_PLAYLIST_SOURCE_CLASS (klass);
 
-	object_class->constructor = rb_generic_player_playlist_source_constructor;
-	object_class->dispose = rb_generic_player_playlist_source_dispose;
-	object_class->finalize = rb_generic_player_playlist_source_finalize;
-	object_class->get_property = rb_generic_player_playlist_source_get_property;
-	object_class->set_property = rb_generic_player_playlist_source_set_property;
+	object_class->constructor = impl_constructor;
+	object_class->dispose = impl_dispose;
+	object_class->finalize = impl_finalize;
+	object_class->get_property = impl_get_property;
+	object_class->set_property = impl_set_property;
 
-	source_class->impl_show_popup = rb_generic_player_playlist_source_show_popup;
+	source_class->impl_show_popup = impl_show_popup;
+	source_class->impl_can_move_to_trash = impl_can_move_to_trash;
+	source_class->impl_move_to_trash = impl_move_to_trash;
 
-	playlist_class->impl_save_contents_to_xml = rb_generic_player_playlist_source_save_to_xml;
-	playlist_class->impl_mark_dirty = rb_generic_player_playlist_source_mark_dirty;
+	playlist_class->impl_save_contents_to_xml = impl_save_to_xml;
+	playlist_class->impl_mark_dirty = impl_mark_dirty;
 
 	g_object_class_install_property (object_class,
 					 PROP_PLAYER_SOURCE,
