@@ -8,7 +8,7 @@
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
  *
- * The Rhythmbox authors hereby grants permission for non-GPL compatible
+ * The Rhythmbox authors hereby grant permission for non-GPL compatible
  * GStreamer plugins to be used and distributed together with GStreamer
  * and Rhythmbox. This permission is above and beyond the permissions granted
  * by the GPL license by which Rhythmbox is covered. If you modify this code
@@ -35,8 +35,8 @@
 #include <gtk/gtk.h>
 #include <glib.h>
 #include <glib-object.h>
-#include <libgnomevfs/gnome-vfs.h>
 
+#include "rb-ipod-helpers.h"
 #include "rb-removable-media-manager.h"
 #include "rb-sourcelist.h"
 #include "rb-source.h"
@@ -47,6 +47,7 @@
 #include "rb-file-helpers.h"
 #include "rb-util.h"
 #include "rb-shell.h"
+#include "rb-stock-icons.h"
 
 
 #define RB_TYPE_IPOD_PLUGIN		(rb_ipod_plugin_get_type ())
@@ -82,7 +83,7 @@ static void impl_activate (RBPlugin *plugin, RBShell *shell);
 static void impl_deactivate (RBPlugin *plugin, RBShell *shell);
 
 static RBSource * create_source_cb (RBRemovableMediaManager *rmm,
-				    GnomeVFSVolume *volume,
+				    GMount *mount,
 				    RBIpodPlugin *plugin);
 static void  rb_ipod_plugin_cmd_rename (GtkAction *action,
 					RBIpodPlugin *plugin);
@@ -92,6 +93,8 @@ static void  rb_ipod_plugin_cmd_playlist_rename (GtkAction *action,
 					RBIpodPlugin *plugin);
 static void  rb_ipod_plugin_cmd_playlist_delete (GtkAction *action,
 					RBIpodPlugin *plugin);
+static void  rb_ipod_plugin_cmd_properties (GtkAction *action,
+					    RBIpodPlugin *plugin);
 
 RB_PLUGIN_REGISTER(RBIpodPlugin, rb_ipod_plugin)
 
@@ -101,7 +104,10 @@ static GtkActionEntry rb_ipod_plugin_actions [] =
 	{ "iPodSourceRename", NULL, N_("_Rename"), NULL,
 	  N_("Rename iPod"),
 	  G_CALLBACK (rb_ipod_plugin_cmd_rename) },
-	{ "iPodSourcePlaylistNew", NULL, N_("_New Playlist"), NULL,
+	{ "iPodProperties", GTK_STOCK_PROPERTIES, N_("_Properties"), NULL,
+	  N_("Display iPod properties"),
+	  G_CALLBACK (rb_ipod_plugin_cmd_properties) },
+	{ "iPodSourcePlaylistNew", RB_STOCK_PLAYLIST_NEW, N_("_New Playlist"), NULL,
 	  N_("Add new playlist to iPod"),
 	  G_CALLBACK (rb_ipod_plugin_cmd_playlist_new) },
 	{ "iPodPlaylistSourceRename", NULL, N_("_Rename"), NULL,
@@ -178,7 +184,7 @@ impl_activate (RBPlugin *bplugin,
 
 	/* watch for new removable media, and cause a rescan */
 	g_signal_connect (G_OBJECT (rmm),
-			  "create-source", G_CALLBACK (create_source_cb),
+			  "create-source-mount", G_CALLBACK (create_source_cb),
 			  plugin);
 
 	/* only scan if we're being loaded after the initial scan has been done */
@@ -223,23 +229,55 @@ rb_ipod_plugin_source_deleted (RBiPodSource *source, RBIpodPlugin *plugin)
 }
 
 static RBSource *
-create_source_cb (RBRemovableMediaManager *rmm, GnomeVFSVolume *volume, RBIpodPlugin *plugin)
+create_source_cb (RBRemovableMediaManager *rmm, GMount *mount, RBIpodPlugin *plugin)
 {
-	if (rb_ipod_is_volume_ipod (volume)) {
-		RBSource *src;
-		src = RB_SOURCE (rb_ipod_source_new (plugin->shell, volume));
-
-		plugin->ipod_sources = g_list_prepend (plugin->ipod_sources, src);
-		g_signal_connect_object (G_OBJECT (src),
-					 "deleted", G_CALLBACK (rb_ipod_plugin_source_deleted),
-					 plugin, 0);
-
-		return src;
+	RBSource *src;
+	if (!rb_ipod_helpers_is_ipod (mount)) {
+		return NULL;
 	}
 
-	return NULL;
+	if (rb_ipod_helpers_needs_init (mount)) {
+		gboolean inited;
+		gchar *builder_file;
+		builder_file = rb_plugin_find_file (RB_PLUGIN (plugin), "ipod-init.ui");
+		inited = rb_ipod_helpers_show_first_time_dialog (mount,
+								 builder_file);
+		g_free (builder_file);
+		if (!inited) {
+			return NULL;
+		}
+	}
+
+	src = RB_SOURCE (rb_ipod_source_new (RB_PLUGIN (plugin),
+					     plugin->shell,
+					     mount));
+
+	plugin->ipod_sources = g_list_prepend (plugin->ipod_sources, src);
+	g_signal_connect_object (G_OBJECT (src),
+				 "deleted", G_CALLBACK (rb_ipod_plugin_source_deleted),
+				 plugin, 0);
+
+	return src;
 }
 
+static void
+rb_ipod_plugin_cmd_properties (GtkAction *action,
+			       RBIpodPlugin *plugin)
+{
+	RBSource *source = NULL;
+
+	g_object_get (G_OBJECT (plugin->shell), 
+		      "selected-source", &source,
+		      NULL);
+	if ((source == NULL) || !RB_IS_IPOD_SOURCE (source)) {
+		g_critical ("got iPodSourceProperties action for non-ipod source");
+		return;
+	}
+
+	rb_ipod_source_show_properties (RB_IPOD_SOURCE (source));
+	g_object_unref (G_OBJECT (source));
+}
+ 
 static void
 rb_ipod_plugin_cmd_rename (GtkAction *action,
 			   RBIpodPlugin *plugin)
