@@ -47,7 +47,6 @@
 #include "rb-debug.h"
 #include "rb-shell.h"
 #include "rb-shell-player.h"
-#include "rb-marshal.h"
 #include "rb-stock-icons.h"
 #include "eel-gconf-extensions.h"
 #include "rb-builder-helpers.h"
@@ -88,6 +87,8 @@ struct _RBStatusIconPluginPrivate
 	guint gconf_notify_id;
 
 	/* configuration */
+	gboolean syncing_actions;
+	gboolean syncing_config_widgets;
 	enum {
 		ICON_NEVER = 0,
 		ICON_WITH_NOTIFY,
@@ -114,9 +115,12 @@ struct _RBStatusIconPluginPrivate
 	GdkPixbuf *notify_pixbuf;
 #ifdef HAVE_NOTIFY
 	NotifyNotification *notification;
+	gboolean notify_supports_actions;
 #endif
 
 	GtkWidget *config_dialog;
+	GtkWidget *notify_combo;
+	GtkWidget *icon_combo;
 
 	RBShellPlayer *shell_player;
 	RBShell *shell;
@@ -186,6 +190,9 @@ toggle_window_cmd (GtkAction *action, RBStatusIconPlugin *plugin)
 static void
 show_window_cmd (GtkAction *action, RBStatusIconPlugin *plugin)
 {
+	if (plugin->priv->syncing_actions)
+		return;
+
 	g_object_set (plugin->priv->shell,
 		      "visibility", gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)),
 		      NULL);
@@ -197,6 +204,9 @@ show_notifications_cmd (GtkAction *action, RBStatusIconPlugin *plugin)
 	gboolean active;
 	int new_mode;
 
+	if (plugin->priv->syncing_actions)
+		return;
+
 	/* we've only got on/off here, so map that to 'never' or 'only when hidden' */
 	active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
 	new_mode = active ? NOTIFY_HIDDEN : NOTIFY_NEVER;
@@ -259,6 +269,8 @@ sync_actions (RBStatusIconPlugin *plugin)
 	GtkAction *action;
 	gboolean visible;
 
+	plugin->priv->syncing_actions = TRUE;
+
 	action = gtk_action_group_get_action (plugin->priv->action_group,
 					      "TrayShowWindow");
 	g_object_get (plugin->priv->shell, "visibility", &visible, NULL);
@@ -277,6 +289,8 @@ sync_actions (RBStatusIconPlugin *plugin)
 	action = gtk_action_group_get_action (plugin->priv->action_group,
 					      "MusicClose");
 	gtk_action_set_visible (action, plugin->priv->icon_mode == ICON_OWNS_WINDOW);
+
+	plugin->priv->syncing_actions = FALSE;
 }
 
 static void
@@ -301,21 +315,43 @@ notification_closed_cb (NotifyNotification *notification,
 	update_status_icon_visibility (plugin, FALSE);
 }
 
+static void
+notification_next_cb (NotifyNotification *notification,
+		      const char *action,
+		      RBStatusIconPlugin *plugin)
+{
+	rb_debug ("notification action: %s", action);
+	rb_shell_player_do_next (plugin->priv->shell_player, NULL);
+}
+
 static void
 do_notify (RBStatusIconPlugin *plugin,
 	   guint timeout,
 	   const char *primary,
 	   const char *secondary,
-	   GdkPixbuf *pixbuf)
+	   GdkPixbuf *pixbuf,
+	   gboolean show_action)
 {
 	const char *icon_name;
 	GError *error = NULL;
 
 	if (notify_is_initted () == FALSE) {
+		GList *caps;
+
 		if (notify_init ("rhythmbox") == FALSE) {
 			g_warning ("libnotify initialization failed");
 			return;
 		}
+
+		/* ask the notification server if it supports actions */
+		caps = notify_get_server_caps ();
+		if (g_list_find_custom (caps, "actions", (GCompareFunc)g_strcmp0) != NULL) {
+			rb_debug ("notification server supports actions");
+			plugin->priv->notify_supports_actions = TRUE;
+		} else {
+			rb_debug ("notification server does not support actions");
+		}
+		rb_list_deep_free (caps);
 	}
 
 	update_status_icon_visibility (plugin, TRUE);
@@ -363,6 +399,16 @@ do_notify (RBStatusIconPlugin *plugin,
 		notify_notification_set_icon_from_pixbuf (plugin->priv->notification, pixbuf);
 	}
 
+	notify_notification_clear_actions (plugin->priv->notification);
+	if (show_action && plugin->priv->notify_supports_actions) {
+		notify_notification_add_action (plugin->priv->notification,
+						"media-next",
+						_("Next"),
+						(NotifyActionCallback) notification_next_cb,
+						plugin,
+						NULL);
+	}
+
 	if (notify_notification_show (plugin->priv->notification, &error) == FALSE) {
 		g_warning ("Failed to send notification (%s): %s", primary, error->message);
 		g_error_free (error);
@@ -430,7 +476,8 @@ notify_playing_entry (RBStatusIconPlugin *plugin, gboolean requested)
 		   PLAYING_ENTRY_NOTIFY_TIME * 1000,
 		   plugin->priv->current_title,
 		   plugin->priv->current_album_and_artist,
-		   plugin->priv->notify_pixbuf);
+		   plugin->priv->notify_pixbuf,
+		   TRUE);
 }
 
 static void
@@ -440,7 +487,7 @@ notify_custom (RBStatusIconPlugin *plugin, guint timeout, const char *primary, c
 		return;
 	}
 
-	do_notify (plugin, timeout, primary, secondary, pixbuf);
+	do_notify (plugin, timeout, primary, secondary, pixbuf, FALSE);
 }
 
 static void
@@ -1030,12 +1077,18 @@ cleanup_status_icon (RBStatusIconPlugin *plugin)
 static void
 notification_config_changed_cb (GtkComboBox *widget, RBStatusIconPlugin *plugin)
 {
+	if (plugin->priv->syncing_config_widgets)
+		return;
+
 	eel_gconf_set_integer (CONF_NOTIFICATION_MODE, gtk_combo_box_get_active (widget));
 }
 
 static void
 status_icon_config_changed_cb (GtkComboBox *widget, RBStatusIconPlugin *plugin)
 {
+	if (plugin->priv->syncing_config_widgets)
+		return;
+
 	eel_gconf_set_integer (CONF_STATUS_ICON_MODE, gtk_combo_box_get_active (widget));
 }
 
@@ -1087,9 +1140,23 @@ config_notify_cb (GConfClient *client, guint connection_id, GConfEntry *entry, R
 		update_status_icon_visibility (plugin, FALSE);	/* maybe should remember if we're notifying.. */
 		sync_actions (plugin);
 
+		if (plugin->priv->icon_combo != NULL) {
+			plugin->priv->syncing_config_widgets = TRUE;
+			gtk_combo_box_set_active (GTK_COMBO_BOX (plugin->priv->icon_combo), plugin->priv->icon_mode);
+			plugin->priv->syncing_config_widgets = FALSE;
+		}
+
 	} else if (g_str_equal (gconf_entry_get_key (entry), CONF_NOTIFICATION_MODE)) {
 		plugin->priv->notify_mode = gconf_value_get_int (gconf_entry_get_value (entry));
 		rb_debug ("notify mode changed to %d", plugin->priv->notify_mode);
+
+		sync_actions (plugin);
+
+		if (plugin->priv->notify_combo != NULL) {
+			plugin->priv->syncing_config_widgets = TRUE;
+			gtk_combo_box_set_active (GTK_COMBO_BOX (plugin->priv->notify_combo), plugin->priv->notify_mode);
+			plugin->priv->syncing_config_widgets = FALSE;
+		}
 	}
 }
 
@@ -1101,8 +1168,6 @@ impl_get_config_widget (RBPlugin *bplugin)
 {
 	RBStatusIconPlugin *plugin;
 	GtkBuilder *builder;
-	GtkComboBox *icon_combo;
-	GtkComboBox *notify_combo;
 	char *builderfile;
 
 	plugin = RB_STATUS_ICON_PLUGIN (bplugin);
@@ -1128,18 +1193,18 @@ impl_get_config_widget (RBPlugin *bplugin)
 	/* connect signals and stuff */
 	g_signal_connect_object (plugin->priv->config_dialog, "response", G_CALLBACK (config_response_cb), plugin, 0);
 
-	icon_combo = GTK_COMBO_BOX (gtk_builder_get_object (builder, "statusiconmode"));
-	notify_combo = GTK_COMBO_BOX (gtk_builder_get_object (builder, "notificationmode"));
-	g_signal_connect_object (notify_combo,
+	plugin->priv->icon_combo = GTK_WIDGET (gtk_builder_get_object (builder, "statusiconmode"));
+	plugin->priv->notify_combo = GTK_WIDGET (gtk_builder_get_object (builder, "notificationmode"));
+	g_signal_connect_object (plugin->priv->notify_combo,
 				 "changed",
 				 G_CALLBACK (notification_config_changed_cb),
 				 plugin, 0);
-	g_signal_connect_object (icon_combo,
+	g_signal_connect_object (plugin->priv->icon_combo,
 				 "changed",
 				 G_CALLBACK (status_icon_config_changed_cb),
 				 plugin, 0);
-	gtk_combo_box_set_active (notify_combo, plugin->priv->notify_mode);
-	gtk_combo_box_set_active (icon_combo, plugin->priv->icon_mode);
+	gtk_combo_box_set_active (GTK_COMBO_BOX (plugin->priv->notify_combo), plugin->priv->notify_mode);
+	gtk_combo_box_set_active (GTK_COMBO_BOX (plugin->priv->icon_combo), plugin->priv->icon_mode);
 
 	g_object_unref (builder);
 	return plugin->priv->config_dialog;
