@@ -8,6 +8,14 @@
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
+ *  The Rhythmbox authors hereby grant permission for non-GPL compatible
+ *  GStreamer plugins to be used and distributed together with GStreamer
+ *  and Rhythmbox. This permission is above and beyond the permissions granted
+ *  by the GPL license by which Rhythmbox is covered. If you modify this code
+ *  you may extend this exception to your version of the code, but you are not
+ *  obligated to do so. If you do not wish to do so, delete this exception
+ *  statement from your version.
+ *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
@@ -26,7 +34,6 @@
 #include <libxml/tree.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <libgnomevfs/gnome-vfs-uri.h>
 
 #include "rb-static-playlist-source.h"
 #include "rb-library-browser.h"
@@ -35,17 +42,26 @@
 #include "rb-stock-icons.h"
 #include "rb-file-helpers.h"
 #include "rb-playlist-xml.h"
+#include "rb-source-search-basic.h"
 
 static GObject *rb_static_playlist_source_constructor (GType type, guint n_construct_properties,
 						       GObjectConstructParam *construct_properties);
 static void rb_static_playlist_source_dispose (GObject *object);
 static void rb_static_playlist_source_finalize (GObject *object);
+static void rb_static_playlist_source_set_property (GObject *object,
+						    guint prop_id,
+						    const GValue *value,
+						    GParamSpec *pspec);
+static void rb_static_playlist_source_get_property (GObject *object,
+						    guint prop_id,
+						    GValue *value,
+						    GParamSpec *pspec);
 
 /* source methods */
 static GList * impl_cut (RBSource *source);
 static void impl_paste (RBSource *asource, GList *entries);
 static void impl_delete (RBSource *source);
-static void impl_search (RBSource *asource, const char *search_text);
+static void impl_search (RBSource *asource, RBSourceSearch *search, const char *cur_text, const char *new_text);
 static void impl_browser_toggled (RBSource *source, gboolean enabled);
 static void impl_reset_filters (RBSource *asource);
 static gboolean impl_receive_drag (RBSource *source, GtkSelectionData *data);
@@ -76,6 +92,9 @@ static void rb_static_playlist_source_row_inserted (GtkTreeModel *model,
 						    GtkTreePath *path,
 						    GtkTreeIter *iter,
 						    RBStaticPlaylistSource *source);
+static gboolean rb_static_playlist_source_filter_entry_drop (RhythmDBQueryModel *model,
+							     RhythmDBEntry *entry,
+							     RBStaticPlaylistSource *source);
 static void rb_static_playlist_source_non_entry_dropped (GtkTreeModel *model,
 							 const char *uri,
 							 int position,
@@ -86,16 +105,18 @@ static void rb_static_playlist_source_rows_reordered (GtkTreeModel *model,
 						      gint *order_map,
 						      RBStaticPlaylistSource *source);
 
-static void search_action_changed (GtkRadioAction *action,
-				   GtkRadioAction *current,
-				   RBShell *shell);
-
 static GtkRadioActionEntry rb_static_playlist_source_radio_actions [] =
 {
-	{ "StaticPlaylistSearchAll", NULL, N_("All"), NULL, N_("Search all fields"), 0 },
-	{ "StaticPlaylistSearchArtists", NULL, N_("Artists"), NULL, N_("Search artists"), 1 },
-	{ "StaticPlaylistSearchAlbums", NULL, N_("Albums"), NULL, N_("Search albums"), 2 },
-	{ "StaticPlaylistSearchTitles", NULL, N_("Titles"), NULL, N_("Search titles"), 3 }
+	{ "StaticPlaylistSearchAll", NULL, N_("All"), NULL, N_("Search all fields"), RHYTHMDB_PROP_SEARCH_MATCH },
+	{ "StaticPlaylistSearchArtists", NULL, N_("Artists"), NULL, N_("Search artists"), RHYTHMDB_PROP_ARTIST_FOLDED },
+	{ "StaticPlaylistSearchAlbums", NULL, N_("Albums"), NULL, N_("Search albums"), RHYTHMDB_PROP_ALBUM_FOLDED },
+	{ "StaticPlaylistSearchTitles", NULL, N_("Titles"), NULL, N_("Search titles"), RHYTHMDB_PROP_TITLE_FOLDED }
+};
+
+enum
+{
+	PROP_0,
+	PROP_BASE_QUERY_MODEL
 };
 
 G_DEFINE_TYPE (RBStaticPlaylistSource, rb_static_playlist_source, RB_TYPE_PLAYLIST_SOURCE)
@@ -112,10 +133,10 @@ typedef struct
 	RBLibraryBrowser *browser;
 	gboolean browser_shown;
 
-	char *search_text;
+	RBSourceSearch *default_search;
+	RhythmDBQuery *search_query;
 
 	GtkActionGroup *action_group;
-	RhythmDBPropType search_prop;
 	gboolean dispose_has_run;
 } RBStaticPlaylistSourcePrivate;
 
@@ -131,6 +152,8 @@ rb_static_playlist_source_class_init (RBStaticPlaylistSourceClass *klass)
 	object_class->constructor = rb_static_playlist_source_constructor;
 	object_class->dispose = rb_static_playlist_source_dispose;
 	object_class->finalize = rb_static_playlist_source_finalize;
+	object_class->set_property = rb_static_playlist_source_set_property;
+	object_class->get_property = rb_static_playlist_source_get_property;
 
 	source_class->impl_can_cut = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_can_paste = (RBSourceFeatureFunc) rb_true_function;
@@ -139,7 +162,6 @@ rb_static_playlist_source_class_init (RBStaticPlaylistSourceClass *klass)
 	source_class->impl_paste = impl_paste;
 	source_class->impl_delete = impl_delete;
 	source_class->impl_receive_drag = impl_receive_drag;
-	source_class->impl_can_search = (RBSourceFeatureFunc) rb_true_function;
 	source_class->impl_search = impl_search;
 	source_class->impl_reset_filters = impl_reset_filters;
 	source_class->impl_can_browse = (RBSourceFeatureFunc) rb_true_function;
@@ -150,6 +172,10 @@ rb_static_playlist_source_class_init (RBStaticPlaylistSourceClass *klass)
 
 	playlist_class->impl_save_contents_to_xml = impl_save_contents_to_xml;
 
+	g_object_class_override_property (object_class,
+					  PROP_BASE_QUERY_MODEL,
+					  "base-query-model");
+
 	g_type_class_add_private (klass, sizeof (RBStaticPlaylistSourcePrivate));
 }
 
@@ -207,6 +233,11 @@ rb_static_playlist_source_dispose (GObject *object)
 		priv->action_group = NULL;
 	}
 
+	if (priv->default_search != NULL) {
+		g_object_unref (priv->default_search);
+		priv->default_search = NULL;
+	}
+
 	G_OBJECT_CLASS (rb_static_playlist_source_parent_class)->dispose (object);
 }
 
@@ -217,7 +248,10 @@ rb_static_playlist_source_finalize (GObject *object)
 
 	rb_debug ("Finalizing static playlist source %p", object);
 
-	g_free (priv->search_text);
+	if (priv->search_query != NULL) {
+		rhythmdb_query_free (priv->search_query);
+		priv->search_query = NULL;
+	}
 
 	G_OBJECT_CLASS (rb_static_playlist_source_parent_class)->finalize (object);
 }
@@ -239,6 +273,10 @@ rb_static_playlist_source_constructor (GType type,
 	priv->base_model = rb_playlist_source_get_query_model (RB_PLAYLIST_SOURCE (psource));
 	g_object_set (priv->base_model, "show-hidden", TRUE, NULL);
 	g_object_ref (priv->base_model);
+	g_signal_connect_object (priv->base_model,
+				 "filter-entry-drop",
+				 G_CALLBACK (rb_static_playlist_source_filter_entry_drop),
+				 source, 0);
 
 	priv->paned = gtk_vpaned_new ();
 
@@ -253,10 +291,13 @@ rb_static_playlist_source_constructor (GType type,
 						    rb_static_playlist_source_radio_actions,
 						    G_N_ELEMENTS (rb_static_playlist_source_radio_actions),
 						    0,
-						    (GCallback)search_action_changed,
-						    shell);
+						    NULL,
+						    NULL);
+		rb_source_search_basic_create_for_actions (priv->action_group,
+							   rb_static_playlist_source_radio_actions,
+							   G_N_ELEMENTS (rb_static_playlist_source_radio_actions));
 	}
-	priv->search_prop = RHYTHMDB_PROP_SEARCH_MATCH;
+	priv->default_search = rb_source_search_basic_new (RHYTHMDB_PROP_SEARCH_MATCH);
 
 	g_object_unref (shell);
 
@@ -297,20 +338,58 @@ rb_static_playlist_source_constructor (GType type,
 }
 
 RBSource *
-rb_static_playlist_source_new (RBShell *shell, const char *name, gboolean local, RhythmDBEntryType entry_type)
+rb_static_playlist_source_new (RBShell *shell, const char *name, const char *sorting_name, gboolean local, RhythmDBEntryType entry_type)
 {
 	if (name == NULL)
 		name = "";
 
+	if (sorting_name == NULL)
+		sorting_name = "";
+
 	return RB_SOURCE (g_object_new (RB_TYPE_STATIC_PLAYLIST_SOURCE,
 					"name", name,
+					"sorting-name", sorting_name,
 					"shell", shell,
 					"is-local", local,
 					"entry-type", entry_type,
 					"source-group", RB_SOURCE_GROUP_PLAYLISTS,
+					"search-type", RB_SOURCE_SEARCH_INCREMENTAL,
 					NULL));
 }
 
+static void
+rb_static_playlist_source_set_property (GObject *object,
+					guint prop_id,
+					const GValue *value,
+					GParamSpec *pspec)
+{
+	/*RBStaticPlaylistSourcePrivate *priv = RB_STATIC_PLAYLIST_SOURCE_GET_PRIVATE (object);*/
+
+	switch (prop_id) {
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
+}
+
+static void
+rb_static_playlist_source_get_property (GObject *object,
+					guint prop_id,
+					GValue *value,
+					GParamSpec *pspec)
+{
+	RBStaticPlaylistSourcePrivate *priv = RB_STATIC_PLAYLIST_SOURCE_GET_PRIVATE (object);
+
+	switch (prop_id) {
+	case PROP_BASE_QUERY_MODEL:
+		g_value_set_object (value, priv->base_model);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
+}
+
 void
 rb_static_playlist_source_load_from_xml (RBStaticPlaylistSource *source, xmlNodePtr node)
 {
@@ -336,6 +415,7 @@ RBSource *
 rb_static_playlist_source_new_from_xml (RBShell *shell, xmlNodePtr node)
 {
 	RBSource *psource = rb_static_playlist_source_new (shell,
+							   NULL,
 							   NULL,
 							   TRUE,
 							   RHYTHMDB_ENTRY_TYPE_SONG);
@@ -393,10 +473,10 @@ impl_reset_filters (RBSource *source)
 	if (rb_library_browser_reset (priv->browser))
 		changed = TRUE;
 
-	if (priv->search_text != NULL) {
+	if (priv->search_query != NULL) {
 		changed = TRUE;
-		g_free (priv->search_text);
-		priv->search_text = NULL;
+		rhythmdb_query_free (priv->search_query);
+		priv->search_query = NULL;
 	}
 
 	if (changed) {
@@ -406,29 +486,22 @@ impl_reset_filters (RBSource *source)
 }
 
 static void
-impl_search (RBSource *source, const char *search_text)
+impl_search (RBSource *source, RBSourceSearch *search, const char *cur_text, const char *new_text)
 {
 	RBStaticPlaylistSourcePrivate *priv = RB_STATIC_PLAYLIST_SOURCE_GET_PRIVATE (source);
-	char *old_search_text = NULL;
+	RhythmDB *db;
 
-	if (search_text != NULL && search_text[0] == '\0')
-		search_text = NULL;
-
-	if (search_text == NULL && priv->search_text == NULL)
-		return;
-	if (search_text != NULL && priv->search_text != NULL
-	    && !strcmp (search_text, priv->search_text))
-		return;
-
-	old_search_text = priv->search_text;
-	if (search_text == NULL) {
-		priv->search_text = NULL;
-	} else {
-		priv->search_text = g_strdup (search_text);
+	if (search == NULL) {
+		search = priv->default_search;
 	}
-	g_free (old_search_text);
 
-	rb_debug ("doing search for \"%s\"", priv->search_text ? priv->search_text : "(NULL)");
+	/* replace our search query */
+	if (priv->search_query != NULL) {
+		rhythmdb_query_free (priv->search_query);
+		priv->search_query = NULL;
+	}
+	db = rb_playlist_source_get_db (RB_PLAYLIST_SOURCE (source));
+	priv->search_query = rb_source_search_create_query (search, db, new_text);
 
 	rb_static_playlist_source_do_query (RB_STATIC_PLAYLIST_SOURCE (source));
 }
@@ -466,10 +539,10 @@ construct_query_from_selection (RBStaticPlaylistSource *source)
 
 	query = g_ptr_array_new();
 
-	if (priv->search_text) {
+	if (priv->search_query != NULL) {
 		rhythmdb_query_append (db,
 				       query,
-				       RHYTHMDB_QUERY_PROP_LIKE, priv->search_prop, priv->search_text,
+				       RHYTHMDB_QUERY_SUBQUERY, priv->search_query,
 				       RHYTHMDB_QUERY_END);
 	}
 
@@ -642,21 +715,12 @@ rb_static_playlist_source_add_location_internal (RBStaticPlaylistSource *source,
 	entry = rhythmdb_entry_lookup_by_location (db, location);
 	if (entry) {
 		RBStaticPlaylistSourcePrivate *priv = RB_STATIC_PLAYLIST_SOURCE_GET_PRIVATE (source);
-		RhythmDBEntryType entry_type;
 
-		g_object_get (source, "entry-type", &entry_type, NULL);
-		if (entry_type != RHYTHMDB_ENTRY_TYPE_INVALID &&
-		    rhythmdb_entry_get_entry_type (entry) != entry_type) {
-			g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
-			rb_debug ("attempting to add an entry of the wrong type to playlist");
-			return;
+		if (_rb_source_check_entry_type (RB_SOURCE (source), entry)) {
+			rhythmdb_entry_ref (entry);
+			rhythmdb_query_model_add_entry (priv->base_model, entry, index);
+			rhythmdb_entry_unref (entry);
 		}
-
-		rhythmdb_entry_ref (entry);
-		rhythmdb_query_model_add_entry (priv->base_model, entry, index);
-		rhythmdb_entry_unref (entry);
-
-		g_boxed_free (RHYTHMDB_TYPE_ENTRY_TYPE, entry_type);
 	}
 
 	rb_playlist_source_add_to_map (psource, location);
@@ -665,12 +729,17 @@ rb_static_playlist_source_add_location_internal (RBStaticPlaylistSource *source,
 }
 
 static gboolean
-rb_static_playlist_source_add_location_cb (const char *uri,
-					   gboolean dir,
-					   RBStaticPlaylistSource *source)
+_add_location_cb (GFile *file,
+		  gboolean dir,
+		  RBStaticPlaylistSource *source)
 {
-	if (!dir)
+	if (!dir) {
+		char *uri;
+
+		uri = g_file_get_uri (file);
 		rb_static_playlist_source_add_location_internal (source, uri, -1);
+		g_free (uri);
+	}
 	return TRUE;	
 }
 
@@ -688,8 +757,8 @@ rb_static_playlist_source_add_location (RBStaticPlaylistSource *source,
 	/* if there is an entry, it won't be a directory */
 	if (entry == NULL && rb_uri_is_directory (location))
 		rb_uri_handle_recursively (location,
-					   (RBUriRecurseFunc) rb_static_playlist_source_add_location_cb,
 					   NULL,
+					   (RBUriRecurseFunc) _add_location_cb,
 					   source);
 	else
 		rb_static_playlist_source_add_location_internal (source, location, index);
@@ -801,6 +870,18 @@ rb_static_playlist_source_rows_reordered (GtkTreeModel *model,
 	rb_playlist_source_mark_dirty (RB_PLAYLIST_SOURCE (source));
 }
 
+static gboolean
+rb_static_playlist_source_filter_entry_drop (RhythmDBQueryModel *model,
+					     RhythmDBEntry *entry, 
+					     RBStaticPlaylistSource *source)
+{
+	if (_rb_source_check_entry_type (RB_SOURCE (source), entry)) {
+		rb_debug ("allowing drop of entry %s", rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION));
+		return TRUE;
+	}
+	rb_debug ("preventing drop of entry %s", rhythmdb_entry_get_string (entry, RHYTHMDB_PROP_LOCATION));
+	return FALSE;
+}
 
 static GList *
 impl_get_search_actions (RBSource *source)
@@ -815,63 +896,6 @@ impl_get_search_actions (RBSource *source)
 	return actions;
 }
 
-static RhythmDBPropType
-search_action_to_prop (GtkAction *action)
-{
-	const char      *name;
-	RhythmDBPropType prop;
-
-	name = gtk_action_get_name (action);
-
-	if (name == NULL) {
-		prop = RHYTHMDB_PROP_SEARCH_MATCH;
-	} else if (strcmp (name, "StaticPlaylistSearchAll") == 0) {
-		prop = RHYTHMDB_PROP_SEARCH_MATCH;
-	} else if (strcmp (name, "StaticPlaylistSearchArtists") == 0) {
-		prop = RHYTHMDB_PROP_ARTIST_FOLDED;
-	} else if (strcmp (name, "StaticPlaylistSearchAlbums") == 0) {
-		prop = RHYTHMDB_PROP_ALBUM_FOLDED;
-	} else if (strcmp (name, "StaticPlaylistSearchTitles") == 0) {
-		prop = RHYTHMDB_PROP_TITLE_FOLDED;
-	} else {
-		prop = RHYTHMDB_PROP_SEARCH_MATCH;
-	}
-
-	return prop;
-}
-
-static void
-search_action_changed (GtkRadioAction  *action,
-		       GtkRadioAction  *current,
-		       RBShell         *shell)
-{
-	RBStaticPlaylistSource *source;
-	RBStaticPlaylistSourcePrivate *priv;
-	gboolean active;
-
-	g_object_get (shell, "selected-source", &source, NULL);
-	if (source == NULL || !RB_IS_STATIC_PLAYLIST_SOURCE (source)) {
-		if (source != NULL)
-			g_object_unref (source);
-		return;
-	}
-
-	priv = RB_STATIC_PLAYLIST_SOURCE_GET_PRIVATE (source);
-
-	active = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (current));
-
-	if (active) {
-		/* update query */
-		priv->search_prop = search_action_to_prop (GTK_ACTION (current));
-		rb_static_playlist_source_do_query (source);
-		rb_source_notify_filter_changed (RB_SOURCE (source));
-	}
-
-	if (source != NULL) {
-		g_object_unref (source);
-	}
-}
-
 static guint
 impl_want_uri (RBSource *source, const char *uri)
 {
@@ -883,3 +907,4 @@ impl_want_uri (RBSource *source, const char *uri)
 
 	return 0;
 }
+
